var version = '4.3.2';

const displayName = 'Preferences Toolbar';
const appName     = 'prefbar';
const author      = 'Manuel Reimer';
const localesToInstall = ["en-US", "de-AT"];

const jarName     = appName + '.jar';

var installedFiles   = [];

function InstallAddon() {
    var err = initInstall(displayName, appName, version);
    if (err != SUCCESS) return SOURCE_DOES_NOT_EXIST;

    var UChrome	= getFolder('Chrome');
    var contentFlag = CONTENT | DELAYED_CHROME;
    var localeFlag  = LOCALE | DELAYED_CHROME;

    var existsInGlobal  = File.exists(getFolder(UChrome, jarName));
    var existsInProfile = File.exists(getFolder(getFolder('Current User', 'chrome'), jarName));
    if (existsInGlobal && existsInProfile) return ALREADY_EXISTS;

    if (existsInProfile ||
       (!existsInGlobal && confirm("Install PrefBar in your profile?\n(you wont have to reinstall when updating the browser)\n\nClick Cancel to install PrefBar in the browser root.\n\n\n" +
                                   "Wollen Sie die PrefBar in ihr Profil installieren?\n(Sie m\u00fcssen dann nicht erneut installieren wenn Sie den Browser aktualisieren)\n\nKlicken Sie Abbrechen um die PrefBar ins Browser-Verzeichnis zu installieren."))) {
        UChrome = getFolder('Current User', 'chrome');
        contentFlag = CONTENT | PROFILE_CHROME;
        localeFlag  = LOCALE | PROFILE_CHROME;
    }

    logComment('initInstall: ' + err);
    setPackageFolder(UChrome);

    addFile(displayName, 'chrome/' + jarName, UChrome, '');
    var folder = getFolder(UChrome, jarName);
    installedFiles.push(folder);

    // Content
    registerChrome(contentFlag, folder, 'content/'+appName+'/');

    // Language packs
    for (s in localesToInstall)
        registerChrome(localeFlag, folder, 'locale/' + localesToInstall[s] + '/'+appName+'/');

    // ****HACK**** (Mozilla Bug 304800,
    //               Mozilla Bug 204271,
    //               or many other bugs caused by XUL.mfasl)
    // Get sure that the file "XUL.mfasl" get's recreated after restart
    // of mozilla
    // Note that this is normally a task that should be done by xpicleanup,
    // but this is corrupted by Mozilla Bug 304797...
    var profiledir = getFolder("Profile");
    var unixxulcache = getFolder(File.dirGetParent(profiledir), "XUL.mfasl");
    var windowsxulcache = getFolder(profiledir, "XUL.mfl");
    if (File.exists(unixxulcache))
        File.remove(unixxulcache);
    if (File.exists(windowsxulcache))
        File.remove(windowsxulcache);
    // ****END HACK****

    err = getLastError();
    if (err != SUCCESS) {
        return err;
    }

    err = performInstall();

    // ****HACK****
    // On linux systems, we get an chrome registration error, if
    // someone tries to update a profile installed PrefBar.
    // We aren't interested in this error, as PrefBar already existed
    // and so the old chrome registration should do the job
//    if (existsInProfile && err == CHROME_REGISTRY_ERROR)
//        err = SUCCESS;
    // ****END HACK****  
    if (err == CHROME_REGISTRY_ERROR)
        alert("PrefBar couldn't be registered! Please restart SeaMonkey and immediately retry the installation after startup.\n\n\n"+
              "PrefBar konnte nicht registriert werden! Bitte starten sie SeaMonkey neu und versuchen Sie die Installation direkt nach dem Starten erneut.");

    // ****HACK**** (Mozilla Bug 304797)
    // This bug makes it impossible to overwrite profile-installed
    // extensions if the user doesn't have write access to the mozilla
    // directory. As there is no way to get around this one 
    // automatically, we have to tell the user what's going on and how
    // to get around the bug manually...
    if (existsInProfile && err == ACCESS_DENIED) {
        alert("You are currently experiencing Mozilla bug 304797 which makes it impossible for us to update your existing PrefBar. The best way to get around this bug is to uninstall your existing PrefBar prior to update (Hold down shift key and right click on PrefBar).\n\n\n" +
              "Sie erleben gerade Mozilla Bug 304797 welcher es uns unm\u00f6glich macht Ihre existierende PrefBar zu aktualisieren. Der beste Weg diesen Bug zu umgehen ist es Ihre existierende PrefBar vor dem Update zu deinstallieren (Umschalttaste gedr\u00fcckt halten und auf die PrefBar rechtsklicken).");
        return err;
    }
    // ****END HACK****

    // Now catch any error, which is no XPInstall bug...
    if (err != SUCCESS && err != REBOOT_NEEDED) {
        return err;
    }

    alert('Ver.' + version + '\n\nInstalled files are:\n' + installedFiles.join('\n') + '\n\n\nInstallation finished.\nPlease restart the browser.\n\n');
    return SUCCESS;
}

// Start this install script
var status = InstallAddon();
if (status != SUCCESS) {
  cancelInstall(status);
  alert("Install failed!  You probably don't have appropriate permissions (write access to the mozilla/chrome directory.)\n\n\n" +
        "Installation fehlgeschlagen! Sie haben vielleicht nicht die n\u00f6tigen Berechtigungen (Schreibrecht im Mozilla/Chrome Verzeichnis).");
}
